@@ -48,6 +48,7 @@
 #define HAVE_TIME_H 1
 #define HAVE_UTIME_H 1
 #define HAVE_GMP_H 1
+#define HAVE_LIBGMP 1
 #define HAVE_TYPEOF 1
 #define HAVE_LONG_LONG 1
 #define HAVE_OFF_T 1
@@ -364,6 +365,7 @@
 #define UCONTEXT_IN_SIGNAL_H 1
 #define DEFINE_MCONTEXT_PTR(mc, uc) mcontext_t *mc = &(uc)->uc_mcontext
 #define USE_ELF 1
+#define HAVE_ELF_H 1
 #define HAVE_ELF_ABI_H 1
 #define DLEXT_MAXLEN 3
 #define DLEXT ".so"
