@@ -2,8 +2,8 @@
 
   rubysig.h -
 
-  $Author: nobu $
-  $Date: 2013-04-05 19:29:38 +0900 (Fri, 05 Apr 2013) $
+  $Author: kosaki $
+  $Date: 2015-10-18 11:10:34 +0900 (Sun, 18 Oct 2015) $
   created at: Wed Aug 16 01:15:38 JST 1995
 
   Copyright (C) 1993-2008 Yukihiro Matsumoto
@@ -12,7 +12,7 @@
 
 #if   defined __GNUC__
 #warning rubysig.h is obsolete
-#elif defined _MSC_VER || defined __BORLANDC__
+#elif defined _MSC_VER
 #pragma message("warning: rubysig.h is obsolete")
 #endif
 
@@ -29,11 +29,6 @@ extern "C" {
 
 RUBY_SYMBOL_EXPORT_BEGIN
 
-struct rb_blocking_region_buffer;
-DEPRECATED(RUBY_EXTERN struct rb_blocking_region_buffer *rb_thread_blocking_region_begin(void));
-DEPRECATED(RUBY_EXTERN void rb_thread_blocking_region_end(struct rb_blocking_region_buffer *));
-#define TRAP_BEG do {struct rb_blocking_region_buffer *__region = rb_thread_blocking_region_begin();
-#define TRAP_END rb_thread_blocking_region_end(__region);} while (0)
 #define RUBY_CRITICAL(statements) do {statements;} while (0)
 #define DEFER_INTS (0)
 #define ENABLE_INTS (1)
