@@ -10,7 +10,7 @@
 
 module RbConfig
   RUBY_VERSION.start_with?("2.4.") or
-    raise "ruby lib version (2.4.3) doesn't match executable version (#{RUBY_VERSION})"
+    raise "ruby lib version (2.4.4) doesn't match executable version (#{RUBY_VERSION})"
 
   # Ruby installed directory.
   TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/2.4/mips64-openbsd")
@@ -21,9 +21,9 @@ module RbConfig
   CONFIG["DESTDIR"] = DESTDIR
   CONFIG["MAJOR"] = "2"
   CONFIG["MINOR"] = "4"
-  CONFIG["TEENY"] = "3"
-  CONFIG["PATCHLEVEL"] = "205"
-  CONFIG["INSTALL"] = '/pobj/ruby-2.4.3/bin/install -c '
+  CONFIG["TEENY"] = "4"
+  CONFIG["PATCHLEVEL"] = "296"
+  CONFIG["INSTALL"] = '/pmfs/ruby-2.4.4/bin/install -c'
   CONFIG["EXEEXT"] = ""
   CONFIG["prefix"] = (TOPDIR || DESTDIR + "/usr/local")
   CONFIG["ruby_install_name"] = "$(RUBY_BASE_NAME)24"
@@ -144,9 +144,9 @@ module RbConfig
   CONFIG["DOT"] = ""
   CONFIG["MAKEDIRS"] = "mkdir -p"
   CONFIG["MKDIR_P"] = "mkdir -p"
-  CONFIG["INSTALL_DATA"] = "/pobj/ruby-2.4.3/bin/install -c  -m 644"
-  CONFIG["INSTALL_SCRIPT"] = "/pobj/ruby-2.4.3/bin/install -c  -m 755"
-  CONFIG["INSTALL_PROGRAM"] = "/pobj/ruby-2.4.3/bin/install -c -s  -m 755"
+  CONFIG["INSTALL_DATA"] = "/pmfs/ruby-2.4.4/bin/install -c -m 644"
+  CONFIG["INSTALL_SCRIPT"] = "/pmfs/ruby-2.4.4/bin/install -c -m 755"
+  CONFIG["INSTALL_PROGRAM"] = "/pmfs/ruby-2.4.4/bin/install -c -s -m 755"
   CONFIG["SET_MAKE"] = ""
   CONFIG["LN_S"] = "ln -s"
   CONFIG["NM"] = "nm"
@@ -183,22 +183,22 @@ module RbConfig
   CONFIG["NACL_SDK_VARIANT"] = ""
   CONFIG["NACL_SDK_ROOT"] = ""
   CONFIG["NACL_TOOLCHAIN"] = ""
-  CONFIG["target_os"] = "openbsd6.3"
+  CONFIG["target_os"] = "openbsd6.4"
   CONFIG["target_vendor"] = "unknown"
   CONFIG["target_cpu"] = "mips64"
-  CONFIG["target"] = "mips64-unknown-openbsd6.3"
-  CONFIG["host_os"] = "openbsd6.3"
+  CONFIG["target"] = "mips64-unknown-openbsd6.4"
+  CONFIG["host_os"] = "openbsd6.4"
   CONFIG["host_vendor"] = "unknown"
   CONFIG["host_cpu"] = "mips64"
-  CONFIG["host"] = "mips64-unknown-openbsd6.3"
+  CONFIG["host"] = "mips64-unknown-openbsd6.4"
   CONFIG["RUBY_VERSION_NAME"] = "$(RUBY_BASE_NAME)-$(ruby_version)"
   CONFIG["RUBYW_BASE_NAME"] = "rubyw"
   CONFIG["RUBY_BASE_NAME"] = "ruby"
-  CONFIG["build_os"] = "openbsd6.3"
+  CONFIG["build_os"] = "openbsd6.4"
   CONFIG["build_vendor"] = "unknown"
   CONFIG["build_cpu"] = "mips64"
-  CONFIG["build"] = "mips64-unknown-openbsd6.3"
-  CONFIG["RUBY_PROGRAM_VERSION"] = "2.4.3"
+  CONFIG["build"] = "mips64-unknown-openbsd6.4"
+  CONFIG["RUBY_PROGRAM_VERSION"] = "2.4.4"
   CONFIG["cxxflags"] = "$(optflags) $(debugflags) $(warnflags)"
   CONFIG["cppflags"] = ""
   CONFIG["cflags"] = "$(optflags) $(debugflags) $(warnflags)"
