@@ -1,3 +1,6 @@
+# frozen_string_literal: false
+require 'io/console/size'
+
 ##
 # Stats printer that prints just the files being documented with a progress
 # bar
@@ -5,15 +8,14 @@
 class RDoc::Stats::Normal < RDoc::Stats::Quiet
 
   def begin_adding # :nodoc:
-    puts "Parsing sources..." if $stdout.tty?
+    puts "Parsing sources..."
+    @last_width = 0
   end
 
   ##
   # Prints a file with a progress bar
 
   def print_file files_so_far, filename
-    return unless $stdout.tty?
-
     progress_bar = sprintf("%3d%% [%2d/%2d]  ",
                            100 * files_so_far / @num_files,
                            files_so_far,
@@ -21,7 +23,7 @@ class RDoc::Stats::Normal < RDoc::Stats::Quiet
 
     # Print a progress bar, but make sure it fits on a single line. Filename
     # will be truncated if necessary.
-    terminal_width = (ENV['COLUMNS'] || 80).to_i
+    terminal_width = IO.console_size[1].to_i.nonzero? || 80
     max_filename_size = terminal_width - progress_bar.size
 
     if filename.size > max_filename_size then
@@ -30,19 +32,21 @@ class RDoc::Stats::Normal < RDoc::Stats::Quiet
       filename[0..2] = "..."
     end
 
-    # Pad the line with whitespaces so that leftover output from the
-    # previous line doesn't show up.
     line = "#{progress_bar}#{filename}"
-    padding = terminal_width - line.size
-    line << (" " * padding) if padding > 0
-
-    $stdout.print("#{line}\r")
+    if $stdout.tty?
+      # Clean the line with whitespaces so that leftover output from the
+      # previous line doesn't show up.
+      $stdout.print("\r" << (" " * @last_width) << ("\b" * @last_width) << "\r") if @last_width && @last_width > 0
+      @last_width = line.size
+      $stdout.print("#{line}\r")
+    else
+      $stdout.puts(line)
+    end
     $stdout.flush
   end
 
   def done_adding # :nodoc:
-    puts if $stdout.tty?
+    puts
   end
 
 end
-
