@@ -1,3 +1,4 @@
+# frozen_string_literal: false
 #--
 # $Release Version: 0.3$
 # $Revision: 1.12 $
@@ -90,7 +91,7 @@ class Tracer
   Tracer::display_thread_id = true
   Tracer::display_c_call = false
 
-  @stdout_mutex = Mutex.new
+  @stdout_mutex = Thread::Mutex.new
 
   # Symbol table used for displaying trace information
   EVENT_SYMBOL = {
